/**
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 - JASS: Java Advanced tranSaction Support
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 - This module was originally developed by 
 -
 -    LSD (Distributed Systems Lab, http://lsd.ls.fi.upm.es/lsd/lsd.htm)
 -    at Universidad Politecnica de Madrid (UPM) as an ObjectWeb Consortium
 -    (http://www.objectweb.org) project.
 -
 -    This project has been partially funded by the European Commission under
 -    the IST programme of V FP grant IST-2001-37126 and by the Spanish
 -    Ministry of Science & Technology (MCyT) grants TIC2002-10376-E and
 -    TIC2001-1586-C03-02
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -  The original code and portions created by LSD are 
 -  Copyright (c) 2004 LSD (UPM)  
 -  All rights reserved.
 -  
 - Redistribution and use in source and binary forms, with or without 
 - modification, are permitted provided that the following conditions are met:
 -
 - -Redistributions of source code must retain the above copyright notice, this
 - list of conditions and the following disclaimer. 
 -
 - -Redistributions in binary form must reproduce the above copyright notice, 
 - this list of conditions and the following disclaimer in the documentation 
 - and/or other materials provided with the distribution. 
 -
 - THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 - AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 - IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 - ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 - LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 - CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 - SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 - INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 - CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 - ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 - POSSIBILITY OF SUCH DAMAGE.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 -  Author: Francisco Perez Sorrosal (frperezs)
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/

package org.objectweb.jass.as;

import java.net.InetAddress;
import java.net.UnknownHostException;

import org.apache.log4j.Logger;

/**
 * Implements the Id Generator service.
 * @author fran
 * Date: Feb 16, 2004
 * org.objectweb.jass.asActivityIdGenerator.java
 */
public class ActivityIdGenerator implements java.io.Serializable {

	private static Logger log = Logger.getLogger(ActivityIdGenerator.class);
	private static ActivityIdGenerator singleton = null;

	// Attributes -------------------------------------------------------------

	private String baseGlobalId;
	private long globalIdNumber = 0;
	private ActivityIdImpl lastId = null;

	// Constructors -----------------------------------------------------------

	/**
	 * Default private constructor
	 */
	private ActivityIdGenerator() {

		try {
			baseGlobalId = InetAddress.getLocalHost().getHostName() + "/";
		} catch (UnknownHostException e) {
			baseGlobalId = "localhost/";
		}
	}

	// My Public --------------------------------------------------------------

	/**
	 * Returns the unique id generator service instance.
	 * @return The unique instance.
	 */
	public static ActivityIdGenerator getSingleton() {

		if (singleton == null)
			singleton = new ActivityIdGenerator();

		return singleton;
	}

	/**
	 * Creates a new activity identifier.
	 * @return the new activity identifier.
	 */
	public ActivityIdImpl newActivityId() {

		String id = baseGlobalId + Long.toString(getNextId());
		byte[] globalId = id.getBytes();
		ActivityIdImpl aid = new ActivityIdImpl(globalId);
		log.debug("New Activity Id Created: " + aid.print());
		lastId = aid;
		
		return aid;
	}

	/**
	 * Returns the last generated activity id.
	 * @return the last generated id.
	 */
	public ActivityIdImpl getLastId() {
		return lastId;
	}
	
	// My Protected -----------------------------------------------------------
		
	// My Private -------------------------------------------------------------
		
	/**
	 * Returns the next id number.
	 * @return the next id number.
	 */
	private synchronized long getNextId() {

		return globalIdNumber++;
	}

}
