/**
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 - JASS: Java Advanced tranSaction Support
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 - This module was originally developed by 
 -
 -    LSD (Distributed Systems Lab, http://lsd.ls.fi.upm.es/lsd/lsd.htm)
 -    at Universidad Politecnica de Madrid (UPM) as an ObjectWeb Consortium
 -    (http://www.objectweb.org) project.
 -
 -    This project has been partially funded by the European Commission under
 -    the IST programme of V FP grant IST-2001-37126 and by the Spanish
 -    Ministry of Science & Technology (MCyT) grants TIC2002-10376-E and
 -    TIC2001-1586-C03-02
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -  The original code and portions created by LSD are 
 -  Copyright (c) 2004 LSD (UPM)  
 -  All rights reserved.
 -  
 - Redistribution and use in source and binary forms, with or without 
 - modification, are permitted provided that the following conditions are met:
 -
 - -Redistributions of source code must retain the above copyright notice, this
 - list of conditions and the following disclaimer. 
 -
 - -Redistributions in binary form must reproduce the above copyright notice, 
 - this list of conditions and the following disclaimer in the documentation 
 - and/or other materials provided with the distribution. 
 -
 - THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 - AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 - IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 - ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 - LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 - CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 - SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 - INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 - CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 - ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 - POSSIBILITY OF SUCH DAMAGE.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 -  Author: Francisco Perez Sorrosal (frperezs)
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/

package org.objectweb.jass.as.jboss;

import javax.naming.Context;
import javax.naming.InitialContext;

import org.jboss.system.ServiceMBeanSupport;
import org.objectweb.jass.as.ActivityIdGenerator;

/**
 * MBean that creates and registers the Activity ID Generator service for the 
 * JBOSS application server.
 * @author fran
 * Date: Feb 12, 2004
 * org.objectweb.jass.as.jbossActivityIdService.java
 */
public class JBOSSActivityIdGeneratorService
	extends ServiceMBeanSupport
	implements JBOSSActivityIdGeneratorServiceMBean {

	// Constants ------------------------------------------------------------------

	public static String JNDI_NAME = "ActivityIdGeneratorService";

	// Attributes -----------------------------------------------------------------

	// The singleton activity id generator instance
	private static ActivityIdGenerator activityIdGenerator =
		ActivityIdGenerator.getSingleton();

	// Public ---------------------------------------------------------------------

	// ServiceMBeanSupport implementation -------------------------------------

	/**
	 * Invoked when this JBOSS MBean is started.
	 */
	public void startService() throws Exception {
		Context ctx = new InitialContext();
		// The service is binded directly
		ctx.bind(JNDI_NAME, activityIdGenerator);
	}

	/**
	 * Invoked when this JBOSS MBean is stopped.
	 */
	public void stopService() {
		try {
			Context ctx = new InitialContext();
			ctx.unbind(JNDI_NAME);
		} catch (Exception e) {
			log.error("Failed to unbind", e);
		}
	}

	// ActivityIdServiceMBean implementation ----------------------------------

	/**
	 * Generates a new activity id by invoking the activity identifier generator
	 * implementation {@link org.objectweb.jass.as.ActivityIdGenerator}.
	 * @return the last activity id created.
	 */
	public String newActivityId() {
		return activityIdGenerator.newActivityId().print();
	}

	public String getLastGeneratedId() {
		return activityIdGenerator.getLastId().print();
	}
}
