/**
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 - JASS: Java Advanced tranSaction Support
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 - This module was originally developed by 
 -
 -    LSD (Distributed Systems Lab, http://lsd.ls.fi.upm.es/lsd/lsd.htm)
 -    at Universidad Politecnica de Madrid (UPM) as an ObjectWeb Consortium
 -    (http://www.objectweb.org) project.
 -
 -    This project has been partially funded by the European Commission under
 -    the IST programme of V FP grant IST-2001-37126 and by the Spanish
 -    Ministry of Science & Technology (MCyT) grants TIC2002-10376-E and
 -    TIC2001-1586-C03-02
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -  The original code and portions created by LSD are 
 -  Copyright (c) 2004 LSD (UPM)  
 -  All rights reserved.
 -  
 - Redistribution and use in source and binary forms, with or without 
 - modification, are permitted provided that the following conditions are met:
 -
 - -Redistributions of source code must retain the above copyright notice, this
 - list of conditions and the following disclaimer. 
 -
 - -Redistributions in binary form must reproduce the above copyright notice, 
 - this list of conditions and the following disclaimer in the documentation 
 - and/or other materials provided with the distribution. 
 -
 - THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 - AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 - IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 - ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 - LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 - CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 - SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 - INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 - CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 - ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 - POSSIBILITY OF SUCH DAMAGE.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 -  Author: Francisco Perez Sorrosal (frperezs)
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/

package org.objectweb.jass.examples.travelagency.ejbs;

import org.objectweb.jass.examples.travelagency.exceptions.DBException;
import org
	.objectweb
	.jass
	.examples
	.travelagency
	.exceptions
	.NotEnoughRoomsException;
import org
	.objectweb
	.jass
	.examples
	.travelagency
	.ejbs
	.compensations
	.ReserveRoomsCompensation;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import javax.activity.opennested.UserOpenNested;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Makes the rooms reservation.
 * It creates a ONT activity through the OpenNested service when methods that
 * perform db changes are invoked (reserveRooms(), unreserveRooms()).   
 * @author fran
 * Date: Feb 16, 2004
 * org.objectweb.jass.examples.travelagency.ejbsHotelBean.java
 */
public class HotelBean implements SessionBean {

	private UserOpenNested uon = null;
	private DataSource ds = null;

	public void ejbCreate() throws javax.ejb.CreateException {
		uon = getUserOpenNested();
		ds = getDataSource();
		System.out.println("Hotel stateless EJB created!!!");
	}

	public void ejbActivate() throws EJBException {
	}

	public void ejbPassivate() throws EJBException {
	}

	public void ejbRemove() throws EJBException {
		uon = null;
		ds = null;
		System.out.println("Hotel stateless EJB removed!!!");
	}

	public void setSessionContext(SessionContext sessionContext)
		throws EJBException {
	}

	/**
	 * Obtains the number of free rooms.
	 * @return - the number of free rooms.
	 */
	public int getFreeRooms() throws DBException {
		Connection c = null;
		Statement stmt = null;
		ResultSet rs = null;
		int rooms = -1;

		try {
			c = ds.getConnection();
			stmt = c.createStatement();
			rs = stmt.executeQuery("SELECT rooms FROM departments");
			if (rs.next()) {
				rooms = rs.getInt(1);
			}
			rs.close();
			stmt.close();
			c.close();
		} catch (SQLException e) {
			throw new DBException();
		}
		return rooms;
	}

	/**
	 * Performs the rooms reservation. If there are not enough rooms, it throws
	 * the NotEnoughRoomsException.
	 * @param nRooms - number of rooms to reserve.
	 * @return 
	 * @throws NotEnoughRoomsException - thown if there are not enough rooms.
	 */
	public int reserveRooms(int nRooms)
		throws NotEnoughRoomsException, DBException {
		Connection c = null;
		Statement stmt = null;
		ResultSet rs = null;
		int result = -1;

		try {
			uon.activityBegin(0);
			c = ds.getConnection();
			stmt = c.createStatement();
			rs = stmt.executeQuery("SELECT rooms FROM departments");
			rs.next();
			int freeRooms = rs.getInt(1);
			if ((freeRooms - nRooms) < 0) {
				rs.close();
				stmt.close();
				c.close();
				uon.activityRollback();
				throw new NotEnoughRoomsException();
			} else {
				stmt.executeUpdate(
					"UPDATE departments SET rooms = " + (freeRooms - nRooms));
				ReserveRoomsCompensation compensation =
					new ReserveRoomsCompensation(this, nRooms);
				uon.activityCommit(compensation);
			}
			rs.close();
			stmt.close();
			c.close();
			result = nRooms;
		} catch (SQLException e) {
			try {
				uon.activityRollback();
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			throw new DBException();
		} catch (NotEnoughRoomsException e) {
			throw e;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}

	/**
	 * Performs the compensation for the reserveRooms() operation.
	 * @param nRooms - the number of rooms to cancel.
	 * @return
	 */
	public int unreserveRooms(int nRooms) throws DBException {
		Connection c = null;
		Statement stmt = null;
		ResultSet rs = null;
		int result = -1;

		try {
			uon.activityBegin(0);
			c = ds.getConnection();
			stmt = c.createStatement();
			rs = stmt.executeQuery("SELECT rooms FROM departments");
			rs.next();
			int freeRooms = rs.getInt(1);
			stmt.executeUpdate(
				"UPDATE departments SET rooms = " + (freeRooms + nRooms));
			uon.activityCommit(null);
			rs.close();
			stmt.close();
			c.close();
			result = nRooms;
		} catch (SQLException e) {
			try {
				uon.activityRollback();
			} catch (Exception e1) {
				e1.printStackTrace();
			}
			throw new DBException();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}

	/**
	* Gets the reference to the UserOpenNested service.
	*/
	private UserOpenNested getUserOpenNested() {
		UserOpenNested uon = null;
		Context ctx = null;

		try {
			ctx = new InitialContext();
			uon = (UserOpenNested) ctx.lookup("UserOpenNested");
		} catch (Exception e) {
			e.printStackTrace();
		}
		return uon;
	}

	/**
		* Gets the Hotel datasource.
		*/
	private DataSource getDataSource() {
		Context ctx = null;
		DataSource ds = null;
		try {
			ctx = new InitialContext();
			ds = (DataSource) ctx.lookup("java:/AdaptHotelDS");
		} catch (NamingException e) {
			e.printStackTrace();
		}
		return ds;
	}
}
