/**
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 - JASS: Java Advanced tranSaction Support
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 - This module was originally developed by 
 -
 -    LSD (Distributed Systems Lab, http://lsd.ls.fi.upm.es/lsd/lsd.htm)
 -    at Universidad Politecnica de Madrid (UPM) as an ObjectWeb Consortium
 -    (http://www.objectweb.org) project.
 -
 -    This project has been partially funded by the European Commission under
 -    the IST programme of V FP grant IST-2001-37126 and by the Spanish
 -    Ministry of Science & Technology (MCyT) grants TIC2002-10376-E and
 -    TIC2001-1586-C03-02
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -  The original code and portions created by LSD are 
 -  Copyright (c) 2004 LSD (UPM)  
 -  All rights reserved.
 -  
 - Redistribution and use in source and binary forms, with or without 
 - modification, are permitted provided that the following conditions are met:
 -
 - -Redistributions of source code must retain the above copyright notice, this
 - list of conditions and the following disclaimer. 
 -
 - -Redistributions in binary form must reproduce the above copyright notice, 
 - this list of conditions and the following disclaimer in the documentation 
 - and/or other materials provided with the distribution. 
 -
 - THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 - AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 - IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 - ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 - LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 - CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 - SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 - INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 - CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 - ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 - POSSIBILITY OF SUCH DAMAGE.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 -
 -  Author: Francisco Perez Sorrosal (frperezs)
 -
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
*/

package org.objectweb.jass.hls.ont.jboss;


import org.objectweb.jass.hls.ont.ONT;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Reference;
import javax.naming.Name;
import javax.naming.spi.ObjectFactory;

import org.jboss.system.ServiceMBeanSupport;

import java.util.Hashtable;

/**
 * MBean that creates the ONT service for the JBOSS application server.
 * @author fran
 * Date: Feb 11, 2004
 * org.objectweb.jass.hls.ont.jbossONTService.java
 */
public class ONTService
        extends ServiceMBeanSupport
        implements ONTServiceMBean,
        ObjectFactory {

// Constants ------------------------------------------------------------------

	// JNDI name to bind the the ONT service in JBOSS 
    public static final String ONT_JNDI_NAME = "UserOpenNested";
	// The ONT service singleton instance
    private static ONT ont = null;

// Public ---------------------------------------------------------------------

    // ServiceMBeanSupport implementation -------------------------------------

	 /**
	  * Invoked when this JBOSS MBean is started.
	  */
    public void startService() throws Exception {

        log.info("Binding the ONT Service...");
        ont = ONT.getSingleton();
        bindRef(ONT_JNDI_NAME, "org.objectweb.jass.hls.ont.ONT");
        log.info("Binding done!!!");
    }

	/**
	 * Invoked when this JBOSS MBean is stopped.
	 */
    public void stopService() {

        try {
            Context ctx = new InitialContext();
            ctx.unbind(ONT_JNDI_NAME);
        } catch (Exception e) {
            log.error("Failed to unbind", e);
        }
    }

    // ObjectFactory implementation -------------------------------------------

	/**
	 * It returns the unique ONT service instance when the service is requested.
	 */
    public Object getObjectInstance(Object obj, Name name, Context nameCtx,
                                    Hashtable environment) throws Exception {
        log.info(Thread.currentThread() + " Obtaining the ONT singleton...");
        return ont;
    }

    // ONTServiceMBean implementation -----------------------------------------

	/**
	 * It returns the singleton ONT instance to the JBOSS-JMX.
	 */
    public ONT getONT() {
        return ont;
    }


// My private -----------------------------------------------------------------
	
	/**
	 * Binds the ONT service implementation class 
	 * (org.objectweb.jass.hls.ont.ONT) with a JNDI name by means 
	 * of a reference.
	 */
    private void bindRef(String jndiName, String className)
            throws Exception {
        Reference ref = new Reference(className, getClass().getName(), null);
        new InitialContext().bind(jndiName, ref);
    }

}